<?php
use WHMCS\Database\Capsule as DB;
define('API_ENDPOINT', 'https://api.synergywholesale.com');
define('MODULE_VERSION', '1.3.0');

function synergywholesale_hosting_renameModule($names_map)
{
    foreach ($names_map as $old => $new) {
        foreach (DB::table('tblproducts')->where('servertype', $old)->select('id')->get() as $pr) {
            DB::table('tblproducts')
                ->where('id', $pr->id)
                ->update(['servertype' => $new]);
        }
    }
}

function synergywholesale_hosting_MetaData()
{
    return [
        'DisplayName' => 'Synergy Wholesale Hosting'
    ];
}

function synergywholesale_hosting_ConfigOptions()
{
    $relid = isset($_POST['id']) ? $_POST['id'] : $_GET['id'];

    synergywholesale_hosting_renameModule(
            ['synergyWholesaleHosting'=>'synergywholesale_hosting']
    );
    
    $check = DB::table('tblservers')
        ->where('name', 'Synergy Wholesale')
        ->count();
    if (!$check) {
        $server_id = DB::table('tblservers')->insertGetId([
            'name' => 'Synergy Wholesale',
            'ipaddress' => '',
            'assignedips' => '',
            'hostname' => '',
            'monthlycost' => '0.00',
            'noc' => '',
            'statusaddress' => '',
            'nameserver1' => '',
            'nameserver1ip' => '',
            'nameserver2' => '',
            'nameserver2ip' => '',
            'nameserver3' => '',
            'nameserver3ip' => '',
            'nameserver4' => '',
            'nameserver4ip' => '',
            'nameserver5' => '',
            'nameserver5ip' => '',
            'maxaccounts' => '0',
            'type' => 'synergywholesale_hosting',
            'username' => '',
            'password' => '',
            'accesshash' => '',
            'secure' => 'on',
            'port' => null,
            'active' => 1,
            'disabled' => 0
        ]);

        $accounts = DB::table('tblhosting')
            ->join('tblproducts', 'tblhosting.packageid', '=', 'tblproducts.id')
            ->where('tblproducts.servertype', 'synergywholesale_hosting')
            ->select('tblhosting.id')
            ->get();

        foreach ($accounts as &$account) {
            $account = $account->id;
        }

        DB::table('tblhosting')
            ->whereIn('id', $accounts)
            ->update(['server' => $server_id]);
    }

    # Should return an array of the module options for each product - maximum of 24
    DB::table('tblproducts')
        ->where('id', $relid)
        ->update(["configoption5" => MODULE_VERSION]);

    $fields = [
        [
            'name' => 'Hosting Id', 
            'type' => 'text'
        ], [
            'name' => 'Server IP Address', 
            'type' => 'text'
        ], [
            'name' => 'Nameserver 1',
            'type' => 'text'
        ], [
            'name' => 'Nameserver 2',
            'type' => 'text'
        ], [
            'name' => 'Nameserver 3',
            'type' => 'text'
        ]
    ];

    foreach ($fields as $field) {
        $fieldExists = DB::table('tblcustomfields')
            ->where([
                ['relid', $relid],
                ['fieldname', $field['name']]
            ])
            ->count();
       if (!$fieldExists) {
        
            DB::table('tblcustomfields')
            ->insert([
                'relid' => $relid,
                'type' => 'product',
                'fieldname' => $field['name'],
                'fieldtype' => $field['type'],
                'fieldoptions' => '',
                'adminonly' => 'on',
                'sortorder' => '0'
            ]);
        }
    }

    return [
        'API Key' => [
            'Type' => 'text',
            'Size' => '60'
        ],
        'Reseller ID' => [
            'Type' => 'text', 
            'Size' => '60'
        ],
        'Hosting Plan' => [
            'FriendlyName' => 'Hosting Plan', 
            'Type' => 'text', 
            'Size' => '60'
        ],
        'Hosting Location' => [
            'FriendlyName' => 'Hosting Location', 
            'Type' => 'dropdown', 
            'Options' => 'NextDC S1 - Sydney Australia', 
            'Size' => '60'
        ],
        'Module Version' => [
            'FriendlyName' => 'Module Version', 
            'Description' => MODULE_VERSION
        ],
        'IP Address' => [
            'FriendlyName' => 'IP Address',
            'Description' => $_SERVER['SERVER_ADDR'] . ' - You\'ll need to provide IP address to access our API'
        ]
    ];
}

function synergywholesale_hosting_AdminCustomButtonArray()
{
    return [
        'Recreate Service' => 'recreate',
        'Synchronize Data' => 'synchronize'
    ];
}

function customValues($params)
{
    $result = DB::table('tblcustomfields')
        ->select('id', 'fieldname')
        ->where('relid', $params['pid'])
        ->get();
    $ids = [];
    foreach ($result as $data) {
        $ids[$data->fieldname] = $data->id;
    }
    unset($result);
    $result = DB::table('tblcustomfieldsvalues')
        ->where('relid', $params['serviceid'])
        ->select('fieldid', 'relid', 'value')
        ->get();
    $values = [];
    foreach ($result as $data) {
        $values[$data->fieldid] = $data->value;
    }
    return [
        'hoid' => $values[$ids['Hosting Id']],
        'server_ip' => $values[$ids['Server IP Address']],
        'ids' => $ids,
        'nameservers' => [
            $values[$ids['Nameserver 1']],
            $values[$ids['Nameserver 2']],
            $values[$ids['Nameserver 3']]
        ]
    ];
}

function synergywholesale_hosting_synchronize($params)
{
    $resellerId = $params['configoption2'];
    $apiKey = $params['configoption1'];
    $customValues = customValues($params);
    $hoid = $customValues['hoid'];

    $updateCustomField = function($service_id, $field_id, $value) {

        $serviceCustomField = DB::table('tblcustomfieldsvalues')
            ->where('fieldid', $field_id)
            ->where('relid', $service_id)
            ->first();
        if ($serviceCustomField) {
            return DB::table('tblcustomfieldsvalues')
                ->where('fieldid', $field_id)
                ->where('relid', $service_id)
                ->update(['value' => $value]);
        } else {
            return DB::table('tblcustomfieldsvalues')->insert([
                'fieldid' => $field_id,
                'relid' => $service_id,
                'value' => $value
            ]);
        }
    };

    $data = [
        'hoid' => $hoid,
        'reason' => 'WHMCS',
        'api_method' => '1',
        'whmcs_ver' => $params['whmcsVersion'],
        'whmcs_mod_ver' => $params['configoption5']
    ];

    $apiResult = synergywholesale_hosting_api($resellerId, $apiKey, 'hostingGetService', $data);
    if ($apiResult->errorMessage == 'Hosting Get Service Completed Successfully') {
        $updateCustomField($params['serviceid'], $customValues['ids']['Server IP Address'], $apiResult->serverIPAddress);


        $updateData = [
            'username' => $apiResult->username,
            'domain' => $apiResult->domain,
            'dedicatedip' => $apiResult->dedicatedIPv4,
            'password' => encrypt($apiResult->password)
        ];

        foreach ($apiResult->nameServers as $index => $nameserver) {
            if ($index >= 3) {
                break;
            }
            
            $field_id = null;
            $fieldName = sprintf('Nameserver %d', $index + 1);
            if (array_key_exists($fieldName, $customValues['ids'])) {
                $field_id = $customValues['ids'][$fieldName];
            }

            if (is_null($field_id)) {
                synergywholesale_hosting_ConfigOptions($params['pid']);
                $customValues = customValues($service_id);
                if (!array_key_exists($fieldName, $customValues['ids'])) {
                    continue;
                }
                $field_id = $customValues['ids'][$fieldName];
            }

            $updateCustomField($params['serviceid'], $field_id, $nameserver);
        }

        DB::table('tblhosting')
                ->where('id', $params['serviceid'])
                ->update($updateData);
        return 'success';
    } else {
        return $apiResult->errorMessage . '. Error code: ' . $apiResult->status;
    }
}

function synergywholesale_hosting_CreateAccount($params)
{
    $domain = $params['domain'];
    $email = $params['clientsdetails']['email'];
    $username = $params['username'];
    $password = $params['password'];
    $resellerId = $params['configoption2'];
    $apiKey = $params['configoption1'];
    $plan = $params['configoption3'];
    $location = $params['configoption4'];
    $customValues = customValues($params);
    $updateCustomField = function($service_id, $field_id, $value) {
        $serviceCustomField = DB::table('tblcustomfieldsvalues')
            ->where('fieldid', $field_id)
            ->where('relid', $service_id)
            ->first();
        if ($serviceCustomField) {
            return DB::table('tblcustomfieldsvalues')
                ->where('fieldid', $field_id)
                ->where('relid', $service_id)
                ->update(['value' => $value]);
        } else {
            return DB::table('tblcustomfieldsvalues')->insert([
                'fieldid' => $field_id,
                'relid' => $service_id,
                'value' => $value
            ]);
        }
    };
    if (strpos($location, 'Melbourne') !== false) {
        $customValues['locationName'] = 'MELBOURNE';
    } else {
        $customValues['locationName'] = 'SYDNEY';
    }
    $data = [
        'planName' => $plan,
        'locationName' => $customValues['locationName'],
        'domain' => $domain,
        'email' => $email,
        'username' => $username,
        'password' => $password,
        'api_method' => 'WHMCS',
        'whmcs_ver' => $params['whmcsVersion'],
        'whmcs_mod_ver' => $params['configoption5']
    ];
    $apiResult = synergywholesale_hosting_api($resellerId, $apiKey, 'hostingPurchaseService', $data);
    if (in_array($apiResult->status, ['OK', 'OK_PENDING'], true)) {
        $apiSyncResult = synergywholesale_hosting_api($resellerId, $apiKey, 'hostingGetService', [
            'hoid' => $apiResult->hoid,
            'reason' => 'WHMCS',
            'api_method' => '1',
            'whmcs_ver' => $params['whmcsVersion']
        ]);
        $fieldsToUpdate = [['field_id' => $customValues['ids']['Hosting Id'], 'value' => $apiResult->hoid]];
        if (isset($apiSyncResult->serverIPAddress))
            $fieldsToUpdate[] = [
                'field_id' => $customValues['ids']['Server IP Address'],
                'value' => $apiSyncResult->serverIPAddress
            ];
        foreach ($fieldsToUpdate as $customFields) {
            $updateCustomField($params['serviceid'], $customFields['field_id'], $customFields['value']);
        }
        if (isset($apiResult->username) && $apiResult->username != '') {
            DB::table('tblhosting')
                    ->where('id', $params['serviceid'])
                    ->update(['username' => $apiResult->username]);
        }
        return 'success';
    } else {
        return $apiResult->errorMessage . '. Error code: ' . $apiResult->status;
    }
}

function synergywholesale_hosting_recreate($params)
{
    $resellerId = $params['configoption2'];
    $apiKey = $params['configoption1'];
    $customValues = customValues($params);
    $hoid = $customValues['hoid'];
    $data = [
        'newPassword' => 'AUTO',
        'hoid' => $hoid,
        'reason' => 'WHMCS',
        'api_method' => '1',
        'whmcs_ver' => $params['whmcsVersion'],
        'whmcs_mod_ver' => $params['configoption5']
    ];
    $apiResult = synergywholesale_hosting_api($resellerId, $apiKey, 'hostingRecreateService', $data);
    if ($apiResult->status == 'OK') {
        DB::table('tblhosting')
                ->where('id', $params['serviceid'])
                ->update(['password'=>encrypt($apiResult->password)]);
    }
    return synergywholesale_hosting_status($apiResult);
}

function synergywholesale_hosting_TerminateAccount($params)
{
    $resellerId = $params['configoption2'];
    $apiKey = $params['configoption1'];
    $customValues = customValues($params);
    $hoid = $customValues['hoid'];
    $data = [
        'hoid' => $hoid,
        'reason' => 'WHMCS',
        'api_method' => '1',
        'whmcs_ver' => $params['whmcsVersion'],
        'whmcs_mod_ver' => $params['configoption5']
    ];
    $apiResult = synergywholesale_hosting_api($resellerId, $apiKey, 'hostingTerminateService', $data);
    return synergywholesale_hosting_status($apiResult);
}

function synergywholesale_hosting_SuspendAccount($params)
{
    $resellerId = $params['configoption2'];
    $apiKey = $params['configoption1'];
    $customValues = customValues($params);
    $hoid = $customValues['hoid'];
    $data = [
        'hoid' => $hoid,
        'reason' => 'WHMCS',
        'api_method' => '1',
        'whmcs_ver' => $params['whmcsVersion'],
        'whmcs_mod_ver' => $params['configoption5']
    ];
    $apiResult = synergywholesale_hosting_api($resellerId, $apiKey, 'hostingSuspendService', $data);
    return synergywholesale_hosting_status($apiResult);
}

function synergywholesale_hosting_UnsuspendAccount($params)
{
    $resellerId = $params['configoption2'];
    $apiKey = $params['configoption1'];
    $customValues = customValues($params);
    $hoid = $customValues['hoid'];
    $data = [
        'hoid' => $hoid,
        'api_method' => '1',
        'whmcs_ver' => $params['whmcsVersion'],
        'whmcs_mod_ver' => $params['configoption5']
    ];
    $apiResult = synergywholesale_hosting_api($resellerId, $apiKey, 'hostingUnsuspendService', $data);
    return synergywholesale_hosting_status($apiResult);
}

function synergywholesale_hosting_ChangePassword($params)
{
    $password = $params['password'];
    $resellerId = $params['configoption2'];
    $apiKey = $params['configoption1'];
    $customValues = customValues($params);
    $hoid = $customValues['hoid'];
    $data = [
        'hoid' => $hoid,
        'newPassword' => $password,
        'api_method' => '1',
        'whmcs_ver' => $params['whmcsVersion'],
        'whmcs_mod_ver' => $params['configoption5']
    ];
    $apiResult = synergywholesale_hosting_api($resellerId, $apiKey, 'hostingChangePassword', $data);
    return synergywholesale_hosting_status($apiResult);
}

function synergywholesale_hosting_api($reseller_id, $apiKey, $action, array $data = [])
{
    try {
        $data = array_merge([
            'resellerID' => $reseller_id, 
            'apiKey' => $apiKey
        ], $data);
        $client = new \SoapClient(null, 
            [
                'location' => API_ENDPOINT, 
                'uri' => '', 
                'trace' => true,
                'exceptions' => true
            ]
        );        
        $result = $client->$action($data);
        logModuleCall('Synergy Hosting', $action, $data, (array)$result);
        return $result;
    } catch (\Exception $e) {
        logModuleCall('Synergy Hosting', $action, $data, [
            'exception' => get_class($e), 
            'message' => $e->getMessage(),
            'trace' => $e->getTraceAsString()
        ]);
    }

    return false;
}

function synergywholesale_hosting_ChangePackage($params)
{
    $resellerId = $params['configoption2'];
    $apiKey = $params['configoption1'];
    $plan = $params['configoption3'];
    $location = $params['configoption4'];
    $customValues = customValues($params);
    if (strpos($location, 'Melbourne') !== false) {
        $customValues['locationName'] = 'MELBOURNE';
    } else {
        $customValues['locationName'] = 'SYDNEY';
    }
    $data = [
        'newPlanName' => $plan,
        'newLocationName' => $customValues['locationName'],
        'hoid' => $customValues['hoid'],
        'api_method' => '1',
        'whmcs_ver' => $params['whmcsVersion'],
        'whmcs_mod_ver' => $params['configoption5']
    ];
    $apiResult = synergywholesale_hosting_api($resellerId, $apiKey, 'hostingChangePackage', $data);
    return synergywholesale_hosting_status($apiResult);
}

function synergywholesale_hosting_status($apiResult)
{
    if (is_null($apiResult))
        return 'Fatal error';
    if ($apiResult->status == 'OK') {
        return 'success';
    } else {
        return $apiResult->errorMessage . '. Error code: ' . $apiResult->status;
    }
}

function synergywholesale_hosting_ClientArea($params)
{
    $resellerId = $params['templatevars']['moduleParams']['configoption2'];
    $apiKey = $params['templatevars']['moduleParams']['configoption1'];
    $customValues = customValues($params);
    $hoid = $customValues['hoid'];
    $data = [
        'hoid' => $hoid,
        'reason' => 'WHMCS',
        'api_method' => '1',
        'whmcs_ver' => $params['whmcsVersion'],
        'whmcs_mod_ver' => $params['templatevars']['moduleParams']['configoption5']
    ];
    $apiResult = synergywholesale_hosting_api($resellerId, $apiKey, 'hostingGetService', $data);
    if($apiResult->dedicatedIPv4 == NULL)
        $apiResult->dedicatedIPv4 = 'Dedicated IP has not been configured';
    if(!isset($apiResult->plan))
        return sprintf('<div class="alert alert-danger">%s</div>', htmlentities($apiResult->errorMessage));
    return [
        'templatefile' => 'clientarea',
        'vars' => [
            'plan' => $apiResult->plan,
            'status' => $apiResult->status,
            'server' => $apiResult->server,
            'dedicatedIP' => $apiResult->dedicatedIPv4,
            'serverIP' => $apiResult->serverIPAddress
        ],
    ];
}

function synergywholesale_hosting_LoginLink()
{
   return '<button type="button" class="btn btn-primary" onclick="runModuleCommand(\'custom\',\'cpanel_admin\')" id="btnLogin_to_cPanel">Login to cPanel</button>';
}

function synergywholesale_hosting_cpanel_admin($params)
{
    $url = synergywholesale_hosting_cpanel_login($params);
    return ($url ? 'window|' . $url : 'Please contact support.');
}

function synergywholesale_hosting_cpanel_client_login($params)
{
    $url = synergywholesale_hosting_cpanel_login($params);
    if ($url) {
        echo sprintf('<script type="text/javascript">window.open("%s","_blank");</script>', $url);
        return 'success';
    } else {
        return 'Please contact support.';
    }
}

function synergywholesale_hosting_cpanel_login($params)
{
    $resellerId = $params['configoption2'];
    $apiKey = $params['configoption1'];
    $customValues = customValues($params);
    $data = [
        'hoid' => $customValues['hoid'],
        'reason' => 'WHMCS',
        'api_method' => '1',
        'whmcs_ver' => $params['whmcsVersion'],
        'whmcs_mod_ver' => $params['templatevars']['moduleParams']['configoption5']
    ];
    $apiResult = synergywholesale_hosting_api($resellerId, $apiKey, 'hostingGetService', $data);
    try {
        return synergywholesale_hosting_getLoginUrl($apiResult->username, $apiResult->password, $apiResult->server);
    } catch (\Exception $e) {
        logModuleCall('Synergy Hosting', $action, $data, ['exception' => get_class($e), 'message' => $e->getMessage(), 'trace' => $e->getTraceAsString()]);
    }
    
    return false;
}

function synergywholesale_hosting_getLoginUrl($user, $pass, $hostname, $service = 'cpanel', $goto = '/') 
{
    $servicePorts = [
        'cpanel' => 2083,
        'whm' => 2087, 
        'webmail' => 2096
    ];
    $port = isset($servicePorts[$service]) ? $servicePorts[$service] : $servicePorts['cpanel'];
    $postFields = [
        'user' => $user,
        'pass' => $pass,
        'goto_uri' => $goto
    ];
    $url = 'https://' . $hostname . ':' . $port;
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $url . '/login');
    curl_setopt($ch, CURLOPT_POST, true);
    curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($postFields));
    curl_setopt($ch, CURLOPT_HTTPHEADER, ['Connection' => 'close']);
    curl_setopt($ch, CURLOPT_HEADER, true);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 30);
    $page = curl_exec($ch);
    curl_close($ch);
    $session = $token = [];
    if (!preg_match('/session=([^\;]+)/', $page, $session)) {
        return false;
    }
    if (!preg_match('|<META HTTP-EQUIV="refresh"[^>]+URL=/(cpsess\d+)/|i', $page, $token)) {
        return false;
    }
    
    return sprintf('%s/%s/login/?session=%s', 
        $url, 
        $token[1], 
        $session[1],
        $goto == '/' ? '' : '&goto_uri=' . urlencode($goto)
    );
}

function synergywholesale_hosting_ClientAreaCustomButtonArray()
{
    return [
        'Login to cPanel' => 'cpanel_client_login',
        'Manage Temporary URL' => 'tempurl',
        'Check Firewall' => 'firewall'
    ];
}

function synergywholesale_hosting_TempUrl($params)
{
    $resellerId = $params['configoption2'];
    $apiKey = $params['configoption1'];
    $customValues = customValues($params);
    $data = [
        'hoid' => $customValues['hoid'],
        'reason' => 'WHMCS',
        'api_method' => '1',
        'whmcs_ver' => $params['whmcsVersion'],
        'whmcs_mod_ver' => $params['templatevars']['moduleParams']['configoption5']
    ];
    $hostingDetails = synergywholesale_hosting_api($resellerId, $apiKey, 'hostingGetService', $data);

    $tempUrl = $hostingDetails->tempUrl;
    $tempUrlAddress = 'http://'.$hostingDetails->server.'/~'.$hostingDetails->username;

    $now = new DateTime("now");
    $tempUrlDate = $hostingDetails->tempUrlDate;
    
    $tempUrlDate = date('Y-m-d', strtotime($tempUrlDate . ' + 28 days'));
    $tempUrlDate = new DateTime($tempUrlDate);
    $dayDiff = date_diff($now, $tempUrlDate);
    $daysRemain = $dayDiff->format('%a');

    if (isset($_POST['module_function']) && $_POST['module_function'] == 'toggle') {
        if ($tempUrl) {
            $apiResult = synergywholesale_hosting_api($resellerId, $apiKey, 'hostingDisableTempUrl', $data);
        }else {
            $apiResult = synergywholesale_hosting_api($resellerId, $apiKey, 'hostingEnableTempUrl', $data);
        }
        if ($apiResult->status == 'OK') {
            $tempUrl = !$tempUrl;
        }
    } else if (isset($_POST['module_function']) && $_POST['module_function'] == 'reset') {
        $apiResult = synergywholesale_hosting_api($resellerId, $apiKey, 'hostingDisableTempUrl', $data);
        $apiResult = synergywholesale_hosting_api($resellerId, $apiKey, 'hostingEnableTempUrl', $data);

        $message = 'Your temporary URL time limit has been reset to 28 days.';
    }
   
    return [
        'templatefile' => 'tempurl',
        'vars' => [
            'server' => $hostingDetails->server,
            'tempUrl' => $tempUrl,
            'domainName' => $hostingDetails->domain,
            'tempUrlAddress' => $tempUrlAddress,
            'daysRemain' => $daysRemain,
            'message'   =>  $message
        ],
    ];    
}

function synergywholesale_hosting_firewall($params)
{
    $resellerId = $params['configoption2'];
    $apiKey = $params['configoption1'];
    $customValues = customValues($params);
    $message = '';
    if (isset($_POST['module_function']) && in_array($_POST['module_function'], ['check', 'unblock_ip'])) {
        if ($_POST['ipopt'] == 'custom' && !empty($_POST['customip'])) {
            $ipAddress = $_POST['customip'];
        }else if($_POST['ipopt'] == 'myIp') {
            $ipAddress = $_POST['myIpAddress'];
        }
        $data = [
            'hoid' => $customValues['hoid'],
            'ipAddress' => $ipAddress,
            'reason' => 'WHMCS',
            'api_method' => '1',
            'whmcs_ver' => $params['whmcsVersion'],
            'whmcs_mod_ver' => $params['templatevars']['moduleParams']['configoption5']
        ];
        if (!empty($ipAddress)) {
            if (!filter_var($ipAddress, FILTER_VALIDATE_IP)) {
                $message = 'IP Address invalid! Please enter a valid IP Address.';
            }else {
                $apiResult = synergywholesale_hosting_api($resellerId, $apiKey, 'hostingCheckFirewall', $data);
                if (!$apiResult->blocked) {
                    $message = 'The IP Address ' . $ipAddress . ' is not blocked!';
                }
            }
        }else {
            $message = 'Please enter an IP Address!';
        }
        $blocked = $apiResult->blocked;
    }
    if (isset($_POST['module_function']) && $_POST['module_function'] == 'unblock_ip') {
        if ($apiResult->status == 'OK') {
            if ($blocked) {
                $apiResult = synergywholesale_hosting_api($resellerId, $apiKey, 'hostingUnblockFirewall', $data);
                $message = 'The IP Address ' . $ipAddress . ' has been unblocked!';
                if ($apiResult->status == 'OK') {
                    $blocked = false;
                }
            }
        }
    }

    return [
        'templatefile' => 'firewall',
        'vars' => [
            'ipAddress' => $ipAddress,
            'blocked'   => $blocked,
            'message'   => $message
        ],
    ];    
}